@@ -488,6 +488,26 @@ wchar_t* CDECL _wcslwr( wchar_t* str )
     return str;
 }
 
+/*********************************************************************
+ *           wcscspn    (MSVCRT.@)
+ */
+size_t __cdecl wcscspn(const wchar_t *str, const wchar_t *reject)
+{
+    const wchar_t *ptr;
+    for (ptr = str; *ptr; ptr++) if (wcschr( reject, *ptr )) break;
+    return ptr - str;
+}
+
+/*********************************************************************
+ *           wcsspn    (MSVCRT.@)
+ */
+size_t __cdecl wcsspn(const wchar_t *str, const wchar_t *accept)
+{
+    const wchar_t *ptr;
+    for (ptr = str; *ptr; ptr++) if (!wcschr( accept, *ptr )) break;
+    return ptr - str;
+}
+
 /*********************************************************************
  *           wcsncmp    (MSVCRT.@)
  */
@@ -899,7 +919,7 @@ int CDECL __stdio_common_vsprintf( unsigned __int64 options, char *str, size_t l
     int ret;
 
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     ret = pf_printf_a(puts_clbk_str_c99_a,
             &ctx, format, (_locale_t)locale, options & UCRTBASE_PRINTF_MASK, arg_clbk_valist, NULL, &valist);
     puts_clbk_str_a(&ctx, 1, &nullbyte);
@@ -1090,7 +1110,7 @@ int CDECL __stdio_common_vsnprintf_s( unsigned __int64 options,
         const char *format, _locale_t locale, va_list valist )
 {
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     return vsnprintf_s_l_opt(str, sizeOfBuffer, count, format, options & UCRTBASE_PRINTF_MASK, locale, valist);
 }
 
@@ -1102,7 +1122,7 @@ int CDECL __stdio_common_vsnwprintf_s( unsigned __int64 options,
         const wchar_t *format, _locale_t locale, va_list valist )
 {
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     return vsnwprintf_s_l_opt(str, sizeOfBuffer, count, format, options & UCRTBASE_PRINTF_MASK, locale, valist);
 }
 
@@ -1124,7 +1144,7 @@ int CDECL __stdio_common_vsprintf_s( unsigned __int64 options,
         _locale_t locale, va_list valist )
 {
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     return vsnprintf_s_l_opt(str, INT_MAX, count, format, options & UCRTBASE_PRINTF_MASK, locale, valist);
 }
 
@@ -1399,7 +1419,7 @@ int CDECL __stdio_common_vswprintf_p( unsigned __int64 options,
         _locale_t locale, va_list valist )
 {
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     return vswprintf_p_l_opt(str, count, format, options & UCRTBASE_PRINTF_MASK, locale, valist);
 }
 #endif
@@ -1512,7 +1532,7 @@ int CDECL __stdio_common_vswprintf( unsigned __int64 options,
     int ret;
 
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     ret = pf_printf_w(puts_clbk_str_c99_w,
             &ctx, format, locale, options & UCRTBASE_PRINTF_MASK, arg_clbk_valist, NULL, &valist);
     puts_clbk_str_w(&ctx, 1, L"");
@@ -1775,7 +1795,7 @@ int CDECL __stdio_common_vsprintf_p(unsigned __int64 options, char *buffer, size
         const char *format, _locale_t locale, va_list args)
 {
     if (options & ~UCRTBASE_PRINTF_MASK)
-        FIXME("options %s not handled\n", wine_dbgstr_longlong(options));
+        FIXME("options %#I64x not handled\n", options);
     return vsprintf_p_l_opt(buffer, length, format, options & UCRTBASE_PRINTF_MASK, locale, args);
 }
 #endif
@@ -2356,41 +2376,41 @@ wchar_t* __cdecl wcsncpy( wchar_t* s1, const wchar_t *s2, size_t n )
 /******************************************************************
  *		wcsncpy_s (MSVCRT.@)
  */
-INT CDECL wcsncpy_s( wchar_t* wcDest, size_t numElement, const wchar_t *wcSrc,
-                            size_t count )
+INT CDECL wcsncpy_s( wchar_t *dst, size_t elem, const wchar_t *src, size_t count )
 {
-    WCHAR *p = wcDest;
+    WCHAR *p = dst;
     BOOL truncate = (count == _TRUNCATE);
 
-    if(!wcDest && !numElement && !count)
-        return 0;
-
-    if (!wcDest || !numElement)
-        return EINVAL;
-
-    if (!wcSrc)
+    if (!count)
     {
-        *wcDest = 0;
-        return count ? EINVAL : 0;
+        if (dst && elem) *dst = 0;
+        return 0;
     }
 
-    while (numElement && count && *wcSrc)
+    if (!MSVCRT_CHECK_PMT(dst != NULL)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(elem != 0)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(src != NULL))
     {
-        *p++ = *wcSrc++;
-        numElement--;
+        *dst = 0;
+        return EINVAL;
+    }
+
+    while (elem && count && *src)
+    {
+        *p++ = *src++;
+        elem--;
         count--;
     }
-    if (!numElement && truncate)
+    if (!elem && truncate)
     {
         *(p-1) = 0;
         return STRUNCATE;
     }
-    else if (!numElement)
+    else if (!elem)
     {
-        *wcDest = 0;
+        *dst = 0;
         return ERANGE;
     }
-
     *p = 0;
     return 0;
 }
@@ -2431,56 +2451,61 @@ wchar_t* __cdecl wcscat( wchar_t *dst, const wchar_t *src )
 }
 
 /*********************************************************************
- *  wcsncat_s (MSVCRT.@)
- *
+ *           wcsncat_s (MSVCRT.@)
  */
-INT CDECL wcsncat_s(wchar_t *dst, size_t elem,
-        const wchar_t *src, size_t count)
+INT CDECL wcsncat_s(wchar_t *dst, size_t elem, const wchar_t *src, size_t count)
 {
-    size_t srclen;
-    wchar_t dststart;
-    INT ret = 0;
+    size_t i, j;
 
     if (!MSVCRT_CHECK_PMT(dst != NULL)) return EINVAL;
     if (!MSVCRT_CHECK_PMT(elem > 0)) return EINVAL;
-    if (!MSVCRT_CHECK_PMT(src != NULL || count == 0)) return EINVAL;
-
-    if (count == 0)
-        return 0;
-
-    for (dststart = 0; dststart < elem; dststart++)
+    if (count == 0) return 0;
+    if (!MSVCRT_CHECK_PMT(src != NULL))
     {
-        if (dst[dststart] == '\0')
-            break;
-    }
-    if (dststart == elem)
-    {
-        MSVCRT_INVALID_PMT("dst[elem] is not NULL terminated\n", EINVAL);
+        *dst = 0;
         return EINVAL;
     }
 
-    if (count == _TRUNCATE)
+    for (i = 0; i < elem; i++) if (!dst[i]) break;
+
+    if (i == elem)
     {
-        srclen = wcslen(src);
-        if (srclen >= (elem - dststart))
+        MSVCRT_INVALID_PMT("dst[elem] is not NULL terminated\n", EINVAL);
+        *dst = 0;
+        return EINVAL;
+    }
+
+    for (j = 0; (j + i) < elem; j++)
+    {
+        if(count == _TRUNCATE && j + i == elem - 1)
         {
-            srclen = elem - dststart - 1;
-            ret = STRUNCATE;
+            dst[j + i] = '\0';
+            return STRUNCATE;
+        }
+        if(j == count || (dst[j + i] = src[j]) == '\0')
+        {
+            dst[j + i] = '\0';
+            return 0;
         }
     }
-    else
-        srclen = min(wcslen(src), count);
-    if (srclen < (elem - dststart))
-    {
-        memcpy(&dst[dststart], src, srclen*sizeof(wchar_t));
-        dst[dststart+srclen] = '\0';
-        return ret;
-    }
+
     MSVCRT_INVALID_PMT("dst[elem] is too small", ERANGE);
     dst[0] = '\0';
     return ERANGE;
 }
 
+/*********************************************************************
+ *           wcsncat    (NTDLL.@)
+ */
+wchar_t * __cdecl wcsncat(wchar_t *s1, const wchar_t *s2, size_t n)
+{
+    wchar_t *ret = s1;
+    while (*s1) s1++;
+    while (n-- > 0) if (!(*s1++ = *s2++)) return ret;
+    *s1 = 0;
+    return ret;
+}
+
 /*********************************************************************
  * wctoint (INTERNAL)
  */
