@@ -791,16 +791,6 @@ int WINAPI MapUriToBrowserEmulationState(DWORD unk1, DWORD unk2, DWORD unk3)
     return 0;
 }
 
-/***********************************************************************
- *           MapBrowserEmulationModeToUserAgent (URLMON.445)
- *    Undocumented, added in IE8
- */
-int WINAPI MapBrowserEmulationModeToUserAgent(DWORD unk1, DWORD unk2)
-{
-    FIXME("stub: %ld %ld\n", unk1, unk2);
-    return 0;
-}
-
 /***********************************************************************
  *           CoInternetGetBrowserProfile (URLMON.446)
  *    Undocumented, added in IE8
