@@ -558,25 +558,6 @@ BOOL unpack_dde_message( HWND hwnd, UINT message, WPARAM *wparam, LPARAM *lparam
     return TRUE;
 }
 
-BOOL process_rawinput_message( MSG *msg, UINT hw_id, const struct hardware_msg_data *msg_data )
-{
-    struct rawinput_thread_data *thread_data = rawinput_thread_data();
-
-    if (msg->message == WM_INPUT_DEVICE_CHANGE)
-        rawinput_update_device_list();
-    else
-    {
-        thread_data->buffer->header.dwSize = RAWINPUT_BUFFER_SIZE;
-        if (!rawinput_from_hardware_message( thread_data->buffer, msg_data )) return FALSE;
-        thread_data->hw_id = hw_id;
-        msg->lParam = (LPARAM)hw_id;
-    }
-
-    msg->pt = point_phys_to_win_dpi( msg->hwnd, msg->pt );
-    return TRUE;
-}
-
-
 /***********************************************************************
  *		SendMessageTimeoutW  (USER32.@)
  */
