@@ -37,12 +37,9 @@ struct user_callbacks
     NTSTATUS (WINAPI *pNtWaitForMultipleObjects)(ULONG,const HANDLE*,BOOLEAN,BOOLEAN,const LARGE_INTEGER*);
     void (CDECL *draw_nc_scrollbar)( HWND hwnd, HDC hdc, BOOL draw_horizontal, BOOL draw_vertical );
     void (CDECL *free_win_ptr)( struct tagWND *win );
-    HMENU (CDECL *get_sys_menu)( HWND hwnd, HMENU popup );
     void (CDECL *notify_ime)( HWND hwnd, UINT param );
     BOOL (CDECL *post_dde_message)( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD dest_tid,
                                     DWORD type );
-    BOOL (CDECL *process_rawinput_message)( MSG *msg, UINT hw_id, const struct hardware_msg_data *msg_data );
-    BOOL (CDECL *rawinput_device_get_usages)(HANDLE handle, USHORT *usage_page, USHORT *usage);
     void (WINAPI *set_standard_scroll_painted)( HWND hwnd, INT bar, BOOL visible );
     BOOL (CDECL *unpack_dde_message)( HWND hwnd, UINT message, WPARAM *wparam, LPARAM *lparam,
                                       void **buffer, size_t size );
@@ -50,7 +47,6 @@ struct user_callbacks
     void (WINAPI *unregister_imm)( HWND hwnd );
     NTSTATUS (CDECL *try_finally)( NTSTATUS (CDECL *func)( void *), void *arg,
                                    void (CALLBACK *finally_func)( BOOL ));
-    struct rawinput_thread_data *(WINAPI *get_rawinput_thread_data)(void);
 };
 
 #define WM_SYSTIMER         0x0118
@@ -207,55 +203,6 @@ enum builtin_winprocs
     NB_BUILTIN_AW_WINPROCS = WINPROC_DESKTOP
 };
 
-/* FIXME: make it private to menu.c */
-
-/* Menu item structure */
-typedef struct menu_item
-{
-    /* ----------- MENUITEMINFO Stuff ----------- */
-    UINT      fType;          /* Item type. */
-    UINT      fState;         /* Item state.  */
-    UINT_PTR  wID;            /* Item id.  */
-    HMENU     hSubMenu;       /* Pop-up menu.  */
-    HBITMAP   hCheckBit;      /* Bitmap when checked.  */
-    HBITMAP   hUnCheckBit;    /* Bitmap when unchecked.  */
-    LPWSTR    text;           /* Item text. */
-    ULONG_PTR dwItemData;     /* Application defined.  */
-    LPWSTR    dwTypeData;     /* depends on fMask */
-    HBITMAP   hbmpItem;       /* bitmap */
-    /* ----------- Wine stuff ----------- */
-    RECT      rect;           /* Item area (relative to the items_rect),
-                               * see MENU_AdjustMenuItemRect(). */
-    UINT      xTab;           /* X position of text after Tab */
-    SIZE      bmpsize;        /* size needed for the HBMMENU_CALLBACK bitmap */
-} MENUITEM;
-
-typedef struct
-{
-    struct user_object obj;
-    WORD        wFlags;       /* Menu flags (MF_POPUP, MF_SYSMENU) */
-    WORD	Width;        /* Width of the whole menu */
-    WORD	Height;       /* Height of the whole menu */
-    UINT        nItems;       /* Number of items in the menu */
-    HWND        hWnd;         /* Window containing the menu */
-    struct menu_item *items;  /* Array of menu items */
-    UINT        FocusedItem;  /* Currently focused item */
-    HWND	hwndOwner;    /* window receiving the messages for ownerdraw */
-    BOOL        bScrolling;   /* Scroll arrows are active */
-    UINT        nScrollPos;   /* Current scroll position */
-    UINT        nTotalHeight; /* Total height of menu items inside menu */
-    RECT        items_rect;   /* Rectangle within which the items lie.  Excludes margins and scroll arrows */
-    LONG        refcount;
-    /* ------------ MENUINFO members ------ */
-    DWORD	dwStyle;	/* Extended menu style */
-    UINT	cyMax;		/* max height of the whole menu, 0 is screen height */
-    HBRUSH	hbrBack;	/* brush for menu background */
-    DWORD	dwContextHelpID;
-    ULONG_PTR	dwMenuData;	/* application defined value */
-    HMENU       hSysMenuOwner;  /* Handle to the dummy sys menu holder */
-    WORD        textOffset;     /* Offset of text when items have both bitmaps and text */
-} POPUPMENU, *LPPOPUPMENU;
-
 /* FIXME: make it private to class.c */
 typedef struct tagWINDOWPROC
 {
