@@ -18,15 +18,21 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #define NONAMELESSSTRUCT
 #define NONAMELESSUNION
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdarg.h>
 #include <string.h>
+#include <dlfcn.h>
 #include <link.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winbase.h"
 #include "winreg.h"
@@ -40,25 +46,20 @@ unsigned int screen_width = 0;
 unsigned int screen_height = 0;
 RECT virtual_screen_rect = { 0, 0, 0, 0 };
 
-MONITORINFOEXW default_monitor =
-{
-    sizeof(default_monitor),    /* cbSize */
-    { 0, 0, 0, 0 },             /* rcMonitor */
-    { 0, 0, 0, 0 },             /* rcWork */
-    MONITORINFOF_PRIMARY,       /* dwFlags */
-    { '\\','\\','.','\\','D','I','S','P','L','A','Y','1',0 }   /* szDevice */
-};
-
 static const unsigned int screen_bpp = 32;  /* we don't support other modes */
 
+static RECT monitor_rc_work;
 static int device_init_done;
+static BOOL force_display_devices_refresh;
+
+PNTAPCFUNC register_window_callback;
 
 typedef struct
 {
     struct gdi_physdev dev;
 } ANDROID_PDEVICE;
 
-static const struct gdi_dc_funcs android_drv_funcs;
+static const struct user_driver_funcs android_drv_funcs;
 
 
 /******************************************************************************
@@ -67,19 +68,78 @@ static const struct gdi_dc_funcs android_drv_funcs;
 void init_monitors( int width, int height )
 {
     static const WCHAR trayW[] = {'S','h','e','l','l','_','T','r','a','y','W','n','d',0};
+    UNICODE_STRING name;
     RECT rect;
-    HWND hwnd = FindWindowW( trayW, NULL );
+    HWND hwnd;
+
+    RtlInitUnicodeString( &name, trayW );
+    hwnd = NtUserFindWindowEx( 0, 0, &name, NULL, 0 );
 
     virtual_screen_rect.right = width;
     virtual_screen_rect.bottom = height;
-    default_monitor.rcMonitor = default_monitor.rcWork = virtual_screen_rect;
+    monitor_rc_work = virtual_screen_rect;
 
-    if (!hwnd || !IsWindowVisible( hwnd )) return;
-    if (!GetWindowRect( hwnd, &rect )) return;
-    if (rect.top) default_monitor.rcWork.bottom = rect.top;
-    else default_monitor.rcWork.top = rect.bottom;
+    if (!hwnd || !NtUserIsWindowVisible( hwnd )) return;
+    if (!NtUserGetWindowRect( hwnd, &rect )) return;
+    if (rect.top) monitor_rc_work.bottom = rect.top;
+    else monitor_rc_work.top = rect.bottom;
     TRACE( "found tray %p %s work area %s\n", hwnd,
-           wine_dbgstr_rect( &rect ), wine_dbgstr_rect( &default_monitor.rcWork ));
+           wine_dbgstr_rect( &rect ), wine_dbgstr_rect( &monitor_rc_work ));
+
+    if (*p_java_vm) /* if we're notified from Java thread, update registry */
+    {
+        UINT32 num_path, num_mode;
+        force_display_devices_refresh = TRUE;
+        /* trigger refresh in win32u */
+        NtUserGetDisplayConfigBufferSizes( QDC_ONLY_ACTIVE_PATHS, &num_path, &num_mode );
+    }
+}
+
+
+/* wrapper for NtCreateKey that creates the key recursively if necessary */
+static HKEY reg_create_key( const WCHAR *name, ULONG name_len )
+{
+    UNICODE_STRING nameW = { name_len, name_len, (WCHAR *)name };
+    OBJECT_ATTRIBUTES attr;
+    NTSTATUS status;
+    HANDLE ret;
+
+    attr.Length = sizeof(attr);
+    attr.RootDirectory = 0;
+    attr.ObjectName = &nameW;
+    attr.Attributes = 0;
+    attr.SecurityDescriptor = NULL;
+    attr.SecurityQualityOfService = NULL;
+
+    status = NtCreateKey( &ret, MAXIMUM_ALLOWED, &attr, 0, NULL, 0, NULL );
+    if (status == STATUS_OBJECT_NAME_NOT_FOUND)
+    {
+        static const WCHAR registry_rootW[] = { '\\','R','e','g','i','s','t','r','y','\\' };
+        DWORD pos = 0, i = 0, len = name_len / sizeof(WCHAR);
+
+        /* don't try to create registry root */
+        if (len > ARRAY_SIZE(registry_rootW) &&
+            !memcmp( name, registry_rootW, sizeof(registry_rootW) ))
+            i += ARRAY_SIZE(registry_rootW);
+
+        while (i < len && name[i] != '\\') i++;
+        if (i == len) return 0;
+        for (;;)
+        {
+            nameW.Buffer = (WCHAR *)name + pos;
+            nameW.Length = (i - pos) * sizeof(WCHAR);
+            status = NtCreateKey( &ret, MAXIMUM_ALLOWED, &attr, 0, NULL, 0, NULL );
+
+            if (attr.RootDirectory) NtClose( attr.RootDirectory );
+            if (status) return 0;
+            if (i == len) break;
+            attr.RootDirectory = ret;
+            while (i < len && name[i] == '\\') i++;
+            pos = i;
+            while (i < len && name[i] != '\\') i++;
+        }
+    }
+    return ret;
 }
 
 
@@ -88,14 +148,26 @@ void init_monitors( int width, int height )
  */
 void set_screen_dpi( DWORD dpi )
 {
-    static const WCHAR dpi_key_name[] = {'S','o','f','t','w','a','r','e','\\','F','o','n','t','s',0};
     static const WCHAR dpi_value_name[] = {'L','o','g','P','i','x','e','l','s',0};
+    static const WCHAR dpi_key_name[] =
+    {
+        '\\','R','e','g','i','s','t','r','y',
+        '\\','M','a','c','h','i','n','e',
+        '\\','S','y','s','t','e','m',
+        '\\','C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t',
+        '\\','H','a','r','d','w','a','r','e',' ','P','r','o','f','i','l','e','s',
+        '\\','C','u','r','r','e','n','t',
+        '\\','S','o','f','t','w','a','r','e',
+        '\\','F','o','n','t','s'
+    };
     HKEY hkey;
 
-    if (!RegCreateKeyW( HKEY_CURRENT_CONFIG, dpi_key_name, &hkey ))
+    if ((hkey = reg_create_key( dpi_key_name, sizeof(dpi_key_name ))))
     {
-        RegSetValueExW( hkey, dpi_value_name, 0, REG_DWORD, (void *)&dpi, sizeof(DWORD) );
-        RegCloseKey( hkey );
+        UNICODE_STRING name;
+        RtlInitUnicodeString( &name, dpi_value_name );
+        NtSetValueKey( hkey, &name, 0, REG_DWORD, &dpi, sizeof(dpi) );
+        NtClose( hkey );
     }
 }
 
@@ -108,7 +180,7 @@ static void fetch_display_metrics(void)
 
     SERVER_START_REQ( get_window_rectangles )
     {
-        req->handle = wine_server_user_handle( GetDesktopWindow() );
+        req->handle = wine_server_user_handle( NtUserGetDesktopWindow() );
         req->relative = COORDS_CLIENT;
         if (!wine_server_call( req ))
         {
@@ -144,7 +216,7 @@ static ANDROID_PDEVICE *create_android_physdev(void)
 
     if (!device_init_done) device_init();
 
-    if (!(physdev = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*physdev) ))) return NULL;
+    if (!(physdev = calloc( 1, sizeof(*physdev) ))) return NULL;
     return physdev;
 }
 
@@ -158,7 +230,7 @@ static BOOL CDECL ANDROID_CreateDC( PHYSDEV *pdev, LPCWSTR device, LPCWSTR outpu
 
     if (!physdev) return FALSE;
 
-    push_dc_driver( pdev, &physdev->dev, &android_drv_funcs );
+    push_dc_driver( pdev, &physdev->dev, &android_drv_funcs.dc_funcs );
     return TRUE;
 }
 
@@ -172,7 +244,7 @@ static BOOL CDECL ANDROID_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
 
     if (!physdev) return FALSE;
 
-    push_dc_driver( pdev, &physdev->dev, &android_drv_funcs );
+    push_dc_driver( pdev, &physdev->dev, &android_drv_funcs.dc_funcs );
     return TRUE;
 }
 
@@ -182,7 +254,7 @@ static BOOL CDECL ANDROID_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
  */
 static BOOL CDECL ANDROID_DeleteDC( PHYSDEV dev )
 {
-    HeapFree( GetProcessHeap(), 0, dev );
+    free( dev );
     return TRUE;
 }
 
@@ -190,8 +262,8 @@ static BOOL CDECL ANDROID_DeleteDC( PHYSDEV dev )
 /***********************************************************************
  *           ANDROID_ChangeDisplaySettingsEx
  */
-LONG CDECL ANDROID_ChangeDisplaySettingsEx( LPCWSTR devname, LPDEVMODEW devmode,
-                                            HWND hwnd, DWORD flags, LPVOID lpvoid )
+LONG ANDROID_ChangeDisplaySettingsEx( LPCWSTR devname, LPDEVMODEW devmode,
+                                      HWND hwnd, DWORD flags, LPVOID lpvoid )
 {
     FIXME( "(%s,%p,%p,0x%08x,%p)\n", debugstr_w( devname ), devmode, hwnd, flags, lpvoid );
     return DISP_CHANGE_SUCCESSFUL;
@@ -199,37 +271,29 @@ LONG CDECL ANDROID_ChangeDisplaySettingsEx( LPCWSTR devname, LPDEVMODEW devmode,
 
 
 /***********************************************************************
- *           ANDROID_GetMonitorInfo
+ *           ANDROID_UpdateDisplayDevices
  */
-BOOL CDECL ANDROID_GetMonitorInfo( HMONITOR handle, LPMONITORINFO info )
+void ANDROID_UpdateDisplayDevices( const struct gdi_device_manager *device_manager,
+                                   BOOL force, void *param )
 {
-    if (handle != (HMONITOR)1)
+    if (force || force_display_devices_refresh)
     {
-        SetLastError( ERROR_INVALID_HANDLE );
-        return FALSE;
+        struct gdi_monitor gdi_monitor =
+        {
+            .rc_monitor = virtual_screen_rect,
+            .rc_work = monitor_rc_work,
+            .state_flags = DISPLAY_DEVICE_ACTIVE | DISPLAY_DEVICE_ATTACHED,
+        };
+        device_manager->add_monitor( &gdi_monitor, param );
+        force_display_devices_refresh = FALSE;
     }
-    info->rcMonitor = default_monitor.rcMonitor;
-    info->rcWork = default_monitor.rcWork;
-    info->dwFlags = default_monitor.dwFlags;
-    if (info->cbSize >= sizeof(MONITORINFOEXW))
-        lstrcpyW( ((MONITORINFOEXW *)info)->szDevice, default_monitor.szDevice );
-    return TRUE;
-}
-
-
-/***********************************************************************
- *           ANDROID_EnumDisplayMonitors
- */
-BOOL CDECL ANDROID_EnumDisplayMonitors( HDC hdc, LPRECT rect, MONITORENUMPROC proc, LPARAM lp )
-{
-    return proc( (HMONITOR)1, 0, &default_monitor.rcMonitor, lp );
 }
 
 
 /***********************************************************************
  *           ANDROID_EnumDisplaySettingsEx
  */
-BOOL CDECL ANDROID_EnumDisplaySettingsEx( LPCWSTR name, DWORD n, LPDEVMODEW devmode, DWORD flags)
+BOOL ANDROID_EnumDisplaySettingsEx( LPCWSTR name, DWORD n, LPDEVMODEW devmode, DWORD flags )
 {
     static const WCHAR dev_name[CCHDEVICENAME] =
         { 'W','i','n','e',' ','A','n','d','r','o','i','d',' ','d','r','i','v','e','r',0 };
@@ -268,132 +332,44 @@ BOOL CDECL ANDROID_EnumDisplaySettingsEx( LPCWSTR name, DWORD n, LPDEVMODEW devm
 /**********************************************************************
  *           ANDROID_wine_get_wgl_driver
  */
-static struct opengl_funcs * CDECL ANDROID_wine_get_wgl_driver( PHYSDEV dev, UINT version )
+static struct opengl_funcs *ANDROID_wine_get_wgl_driver( UINT version )
 {
-    struct opengl_funcs *ret;
-
-    if (!(ret = get_wgl_driver( version )))
-    {
-        dev = GET_NEXT_PHYSDEV( dev, wine_get_wgl_driver );
-        ret = dev->funcs->wine_get_wgl_driver( dev, version );
-    }
-    return ret;
+    return get_wgl_driver( version );
 }
 
 
-static const struct gdi_dc_funcs android_drv_funcs =
+static const struct user_driver_funcs android_drv_funcs =
 {
-    NULL,                               /* pAbortDoc */
-    NULL,                               /* pAbortPath */
-    NULL,                               /* pAlphaBlend */
-    NULL,                               /* pAngleArc */
-    NULL,                               /* pArc */
-    NULL,                               /* pArcTo */
-    NULL,                               /* pBeginPath */
-    NULL,                               /* pBlendImage */
-    NULL,                               /* pChord */
-    NULL,                               /* pCloseFigure */
-    ANDROID_CreateCompatibleDC,         /* pCreateCompatibleDC */
-    ANDROID_CreateDC,                   /* pCreateDC */
-    ANDROID_DeleteDC,                   /* pDeleteDC */
-    NULL,                               /* pDeleteObject */
-    NULL,                               /* pEllipse */
-    NULL,                               /* pEndDoc */
-    NULL,                               /* pEndPage */
-    NULL,                               /* pEndPath */
-    NULL,                               /* pEnumFonts */
-    NULL,                               /* pExtEscape */
-    NULL,                               /* pExtFloodFill */
-    NULL,                               /* pExtTextOut */
-    NULL,                               /* pFillPath */
-    NULL,                               /* pFillRgn */
-    NULL,                               /* pFontIsLinked */
-    NULL,                               /* pFrameRgn */
-    NULL,                               /* pGetBoundsRect */
-    NULL,                               /* pGetCharABCWidths */
-    NULL,                               /* pGetCharABCWidthsI */
-    NULL,                               /* pGetCharWidth */
-    NULL,                               /* pGetCharWidthInfo */
-    NULL,                               /* pGetDeviceCaps */
-    NULL,                               /* pGetDeviceGammaRamp */
-    NULL,                               /* pGetFontData */
-    NULL,                               /* pGetFontRealizationInfo */
-    NULL,                               /* pGetFontUnicodeRanges */
-    NULL,                               /* pGetGlyphIndices */
-    NULL,                               /* pGetGlyphOutline */
-    NULL,                               /* pGetICMProfile */
-    NULL,                               /* pGetImage */
-    NULL,                               /* pGetKerningPairs */
-    NULL,                               /* pGetNearestColor */
-    NULL,                               /* pGetOutlineTextMetrics */
-    NULL,                               /* pGetPixel */
-    NULL,                               /* pGetSystemPaletteEntries */
-    NULL,                               /* pGetTextCharsetInfo */
-    NULL,                               /* pGetTextExtentExPoint */
-    NULL,                               /* pGetTextExtentExPointI */
-    NULL,                               /* pGetTextFace */
-    NULL,                               /* pGetTextMetrics */
-    NULL,                               /* pGradientFill */
-    NULL,                               /* pInvertRgn */
-    NULL,                               /* pLineTo */
-    NULL,                               /* pMoveTo */
-    NULL,                               /* pPaintRgn */
-    NULL,                               /* pPatBlt */
-    NULL,                               /* pPie */
-    NULL,                               /* pPolyBezier */
-    NULL,                               /* pPolyBezierTo */
-    NULL,                               /* pPolyDraw */
-    NULL,                               /* pPolyPolygon */
-    NULL,                               /* pPolyPolyline */
-    NULL,                               /* pPolylineTo */
-    NULL,                               /* pPutImage */
-    NULL,                               /* pRealizeDefaultPalette */
-    NULL,                               /* pRealizePalette */
-    NULL,                               /* pRectangle */
-    NULL,                               /* pResetDC */
-    NULL,                               /* pRoundRect */
-    NULL,                               /* pSelectBitmap */
-    NULL,                               /* pSelectBrush */
-    NULL,                               /* pSelectFont */
-    NULL,                               /* pSelectPen */
-    NULL,                               /* pSetBkColor */
-    NULL,                               /* pSetBoundsRect */
-    NULL,                               /* pSetDCBrushColor */
-    NULL,                               /* pSetDCPenColor */
-    NULL,                               /* pSetDIBitsToDevice */
-    NULL,                               /* pSetDeviceClipping */
-    NULL,                               /* pSetDeviceGammaRamp */
-    NULL,                               /* pSetPixel */
-    NULL,                               /* pSetTextColor */
-    NULL,                               /* pStartDoc */
-    NULL,                               /* pStartPage */
-    NULL,                               /* pStretchBlt */
-    NULL,                               /* pStretchDIBits */
-    NULL,                               /* pStrokeAndFillPath */
-    NULL,                               /* pStrokePath */
-    NULL,                               /* pUnrealizePalette */
-    NULL,                               /* pD3DKMTCheckVidPnExclusiveOwnership */
-    NULL,                               /* pD3DKMTSetVidPnSourceOwner */
-    ANDROID_wine_get_wgl_driver,        /* wine_get_wgl_driver */
-    NULL,                               /* wine_get_vulkan_driver */
-    GDI_PRIORITY_GRAPHICS_DRV           /* priority */
+    .dc_funcs.pCreateCompatibleDC = ANDROID_CreateCompatibleDC,
+    .dc_funcs.pCreateDC = ANDROID_CreateDC,
+    .dc_funcs.pDeleteDC = ANDROID_DeleteDC,
+    .dc_funcs.priority = GDI_PRIORITY_GRAPHICS_DRV,
+
+    .pGetKeyNameText = ANDROID_GetKeyNameText,
+    .pMapVirtualKeyEx = ANDROID_MapVirtualKeyEx,
+    .pVkKeyScanEx = ANDROID_VkKeyScanEx,
+    .pSetCursor = ANDROID_SetCursor,
+    .pChangeDisplaySettingsEx = ANDROID_ChangeDisplaySettingsEx,
+    .pEnumDisplaySettingsEx = ANDROID_EnumDisplaySettingsEx,
+    .pUpdateDisplayDevices = ANDROID_UpdateDisplayDevices,
+    .pCreateWindow = ANDROID_CreateWindow,
+    .pDesktopWindowProc = ANDROID_DesktopWindowProc,
+    .pDestroyWindow = ANDROID_DestroyWindow,
+    .pMsgWaitForMultipleObjectsEx = ANDROID_MsgWaitForMultipleObjectsEx,
+    .pSetCapture = ANDROID_SetCapture,
+    .pSetLayeredWindowAttributes = ANDROID_SetLayeredWindowAttributes,
+    .pSetParent = ANDROID_SetParent,
+    .pSetWindowRgn = ANDROID_SetWindowRgn,
+    .pSetWindowStyle = ANDROID_SetWindowStyle,
+    .pShowWindow = ANDROID_ShowWindow,
+    .pUpdateLayeredWindow = ANDROID_UpdateLayeredWindow,
+    .pWindowMessage = ANDROID_WindowMessage,
+    .pWindowPosChanging = ANDROID_WindowPosChanging,
+    .pWindowPosChanged = ANDROID_WindowPosChanged,
+    .pwine_get_wgl_driver = ANDROID_wine_get_wgl_driver,
 };
 
 
-/******************************************************************************
- *           ANDROID_get_gdi_driver
- */
-const struct gdi_dc_funcs * CDECL ANDROID_get_gdi_driver( unsigned int version )
-{
-    if (version != WINE_GDI_DRIVER_VERSION)
-    {
-        ERR( "version mismatch, gdi32 wants %u but wineandroid has %u\n", version, WINE_GDI_DRIVER_VERSION );
-        return NULL;
-    }
-    return &android_drv_funcs;
-}
-
-
 static const JNINativeMethod methods[] =
 {
     { "wine_desktop_changed", "(II)V", desktop_changed },
@@ -586,15 +562,21 @@ JavaVM **p_java_vm = NULL;
 jobject *p_java_object = NULL;
 unsigned short *p_java_gdt_sel = NULL;
 
-static BOOL process_attach(void)
+static NTSTATUS CDECL unix_call( enum android_funcs code, void *params );
+NTSTATUS (WINAPI *pNtWaitForMultipleObjects)( ULONG,const HANDLE*,BOOLEAN,
+                                              BOOLEAN,const LARGE_INTEGER* );
+
+static HRESULT android_init( void *arg )
 {
+    struct init_params *params = arg;
+    pthread_mutexattr_t attr;
     jclass class;
     jobject object;
     JNIEnv *jni_env;
     JavaVM *java_vm;
     void *ntdll;
 
-    if (!(ntdll = dlopen( "ntdll.so", RTLD_NOW ))) return FALSE;
+    if (!(ntdll = dlopen( "ntdll.so", RTLD_NOW ))) return STATUS_UNSUCCESSFUL;
 
     p_java_vm = dlsym( ntdll, "java_vm" );
     p_java_object = dlsym( ntdll, "java_object" );
@@ -604,6 +586,14 @@ static BOOL process_attach(void)
 
     load_hardware_libs();
 
+    pthread_mutexattr_init( &attr );
+    pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
+    pthread_mutex_init( &drawable_mutex, &attr );
+    pthread_mutex_init( &win_data_mutex, &attr );
+    pthread_mutexattr_destroy( &attr );
+
+    register_window_callback = params->register_window_callback;
+
     if ((java_vm = *p_java_vm))  /* running under Java */
     {
 #ifdef __i386__
@@ -620,19 +610,28 @@ static BOOL process_attach(void)
         __asm__( "mov %0,%%fs" :: "r" (old_fs) );
 #endif
     }
-    return TRUE;
+    __wine_set_user_driver( &android_drv_funcs, WINE_GDI_DRIVER_VERSION );
+    pNtWaitForMultipleObjects = params->pNtWaitForMultipleObjects;
+    params->unix_call = unix_call;
+    return STATUS_SUCCESS;
 }
 
-/***********************************************************************
- *       dll initialisation routine
- */
-BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
+const unixlib_entry_t __wine_unix_call_funcs[] =
 {
-    switch (reason)
-    {
-    case DLL_PROCESS_ATTACH:
-        DisableThreadLibraryCalls( inst );
-        return process_attach();
-    }
-    return TRUE;
+    android_create_desktop,
+    android_dispatch_ioctl,
+    android_init,
+    android_java_init,
+    android_java_uninit,
+    android_register_window,
+};
+
+
+C_ASSERT( ARRAYSIZE(__wine_unix_call_funcs) == unix_funcs_count );
+
+
+/* FIXME: Use __wine_unix_call instead */
+static NTSTATUS CDECL unix_call( enum android_funcs code, void *params )
+{
+    return __wine_unix_call_funcs[code]( params );
 }
