@@ -37,7 +37,9 @@ struct wg_format
         WG_MAJOR_TYPE_UNKNOWN,
         WG_MAJOR_TYPE_VIDEO,
         WG_MAJOR_TYPE_AUDIO,
+        WG_MAJOR_TYPE_MPEG1_AUDIO,
         WG_MAJOR_TYPE_WMA,
+        WG_MAJOR_TYPE_H264,
     } major_type;
 
     union
@@ -66,6 +68,7 @@ struct wg_format
             } format;
             int32_t width, height;
             uint32_t fps_n, fps_d;
+            RECT padding;
         } video;
         struct
         {
@@ -79,10 +82,6 @@ struct wg_format
                 WG_AUDIO_FORMAT_S32LE,
                 WG_AUDIO_FORMAT_F32LE,
                 WG_AUDIO_FORMAT_F64LE,
-
-                WG_AUDIO_FORMAT_MPEG1_LAYER1,
-                WG_AUDIO_FORMAT_MPEG1_LAYER2,
-                WG_AUDIO_FORMAT_MPEG1_LAYER3,
             } format;
 
             uint32_t channels;
@@ -90,6 +89,12 @@ struct wg_format
             uint32_t rate;
         } audio;
         struct
+        {
+            uint32_t layer;
+            uint32_t rate;
+            uint32_t channels;
+        } mpeg1_audio;
+        struct
         {
             uint32_t version;
             uint32_t bitrate;
@@ -100,9 +105,36 @@ struct wg_format
             uint32_t codec_data_len;
             unsigned char codec_data[64];
         } wma;
+        struct
+        {
+            int32_t width, height;
+            uint32_t fps_n, fps_d;
+            uint32_t profile;
+            uint32_t level;
+        } h264;
     } u;
 };
 
+enum wg_sample_flag
+{
+    WG_SAMPLE_FLAG_INCOMPLETE = 1,
+    WG_SAMPLE_FLAG_HAS_PTS = 2,
+    WG_SAMPLE_FLAG_HAS_DURATION = 4,
+    WG_SAMPLE_FLAG_SYNC_POINT = 8,
+};
+
+struct wg_sample
+{
+    /* timestamp and duration are in 100-nanosecond units. */
+    UINT64 pts;
+    UINT64 duration;
+    LONG refcount; /* unix refcount */
+    UINT32 flags;
+    UINT32 max_size;
+    UINT32 size;
+    BYTE *data;
+};
+
 struct wg_parser_buffer
 {
     /* pts and duration are in 100-nanosecond units. */
@@ -216,6 +248,21 @@ struct wg_transform_create_params
     const struct wg_format *output_format;
 };
 
+struct wg_transform_push_data_params
+{
+    struct wg_transform *transform;
+    struct wg_sample *sample;
+    HRESULT result;
+};
+
+struct wg_transform_read_data_params
+{
+    struct wg_transform *transform;
+    struct wg_sample *sample;
+    struct wg_format *format;
+    HRESULT result;
+};
+
 enum unix_funcs
 {
     unix_wg_parser_create,
@@ -244,6 +291,9 @@ enum unix_funcs
 
     unix_wg_transform_create,
     unix_wg_transform_destroy,
+
+    unix_wg_transform_push_data,
+    unix_wg_transform_read_data,
 };
 
 #endif /* __WINE_WINEGSTREAMER_UNIXLIB_H */
