@@ -19,6 +19,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #include "config.h"
 
 #include "macdrv.h"
@@ -47,15 +51,7 @@ static int device_data_valid;   /* do the above variables have up-to-date values
 
 int retina_on = FALSE;
 
-static CRITICAL_SECTION device_data_section;
-static CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &device_data_section,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": device_data_section") }
-};
-static CRITICAL_SECTION device_data_section = { &critsect_debug, -1, 0, 0, 0, 0 };
-
+static pthread_mutex_t device_data_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static const struct user_driver_funcs macdrv_funcs;
 
@@ -90,7 +86,7 @@ CGRect macdrv_get_desktop_rect(void)
 {
     CGRect ret;
 
-    EnterCriticalSection(&device_data_section);
+    pthread_mutex_lock(&device_data_mutex);
 
     if (!device_data_valid)
     {
@@ -99,7 +95,7 @@ CGRect macdrv_get_desktop_rect(void)
     }
     ret = desktop_rect;
 
-    LeaveCriticalSection(&device_data_section);
+    pthread_mutex_unlock(&device_data_mutex);
 
     TRACE("%s\n", wine_dbgstr_cgrect(ret));
 
@@ -151,9 +147,9 @@ static void device_init(void)
 
 void macdrv_reset_device_metrics(void)
 {
-    EnterCriticalSection(&device_data_section);
+    pthread_mutex_lock(&device_data_mutex);
     device_data_valid = FALSE;
-    LeaveCriticalSection(&device_data_section);
+    pthread_mutex_unlock(&device_data_mutex);
 }
 
 
@@ -161,11 +157,11 @@ static MACDRV_PDEVICE *create_mac_physdev(void)
 {
     MACDRV_PDEVICE *physDev;
 
-    EnterCriticalSection(&device_data_section);
+    pthread_mutex_lock(&device_data_mutex);
     if (!device_data_valid) device_init();
-    LeaveCriticalSection(&device_data_section);
+    pthread_mutex_unlock(&device_data_mutex);
 
-    if (!(physDev = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*physDev)))) return NULL;
+    if (!(physDev = calloc(1, sizeof(*physDev)))) return NULL;
 
     return physDev;
 }
@@ -215,7 +211,7 @@ static BOOL CDECL macdrv_DeleteDC(PHYSDEV dev)
 
     TRACE("hdc %p\n", dev->hdc);
 
-    HeapFree(GetProcessHeap(), 0, physDev);
+    free(physDev);
     return TRUE;
 }
 
@@ -227,7 +223,7 @@ static INT CDECL macdrv_GetDeviceCaps(PHYSDEV dev, INT cap)
 {
     INT ret;
 
-    EnterCriticalSection(&device_data_section);
+    pthread_mutex_lock(&device_data_mutex);
 
     if (!device_data_valid) device_init();
 
@@ -245,7 +241,7 @@ static INT CDECL macdrv_GetDeviceCaps(PHYSDEV dev, INT cap)
     case HORZRES:
     case VERTRES:
     default:
-        LeaveCriticalSection(&device_data_section);
+        pthread_mutex_unlock(&device_data_mutex);
         dev = GET_NEXT_PHYSDEV( dev, pGetDeviceCaps );
         ret = dev->funcs->pGetDeviceCaps( dev, cap );
         if ((cap == HORZRES || cap == VERTRES) && retina_on)
@@ -255,7 +251,7 @@ static INT CDECL macdrv_GetDeviceCaps(PHYSDEV dev, INT cap)
 
     TRACE("cap %d -> %d\n", cap, ret);
 
-    LeaveCriticalSection(&device_data_section);
+    pthread_mutex_unlock(&device_data_mutex);
     return ret;
 }
 
@@ -268,16 +264,15 @@ static const struct user_driver_funcs macdrv_funcs =
     .dc_funcs.pGetDeviceCaps = macdrv_GetDeviceCaps,
     .dc_funcs.pGetDeviceGammaRamp = macdrv_GetDeviceGammaRamp,
     .dc_funcs.pSetDeviceGammaRamp = macdrv_SetDeviceGammaRamp,
-    .dc_funcs.wine_get_wgl_driver = macdrv_wine_get_wgl_driver,
-    .dc_funcs.wine_get_vulkan_driver = macdrv_wine_get_vulkan_driver,
     .dc_funcs.priority = GDI_PRIORITY_GRAPHICS_DRV,
 
     .pActivateKeyboardLayout = macdrv_ActivateKeyboardLayout,
     .pBeep = macdrv_Beep,
     .pChangeDisplaySettingsEx = macdrv_ChangeDisplaySettingsEx,
     .pClipCursor = macdrv_ClipCursor,
+    .pClipboardWindowProc = macdrv_ClipboardWindowProc,
     .pCreateDesktopWindow = macdrv_CreateDesktopWindow,
-    .pCreateWindow = macdrv_CreateWindow,
+    .pDesktopWindowProc = macdrv_DesktopWindowProc,
     .pDestroyCursorIcon = macdrv_DestroyCursorIcon,
     .pDestroyWindow = macdrv_DestroyWindow,
     .pEnumDisplaySettingsEx = macdrv_EnumDisplaySettingsEx,
@@ -309,6 +304,8 @@ static const struct user_driver_funcs macdrv_funcs =
     .pWindowMessage = macdrv_WindowMessage,
     .pWindowPosChanged = macdrv_WindowPosChanged,
     .pWindowPosChanging = macdrv_WindowPosChanging,
+    .pwine_get_vulkan_driver = macdrv_wine_get_vulkan_driver,
+    .pwine_get_wgl_driver = macdrv_wine_get_wgl_driver,
 };
 
 
