@@ -21,6 +21,7 @@
 #include <assert.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <pthread.h>
 #include <libusb.h>
 
 #include "ntstatus.h"
@@ -36,9 +37,11 @@
 #include "wine/list.h"
 #include "wine/unicode.h"
 
+#include "unixlib.h"
+
 WINE_DEFAULT_DEBUG_CHANNEL(wineusb);
 
-#if defined(__i386__) && !defined(_WIN32)
+#ifdef __ASM_USE_FASTCALL_WRAPPER
 
 extern void * WINAPI wrap_fastcall_func1( void *func, const void *a );
 __ASM_STDCALL_FUNC( wrap_fastcall_func1, 8,
@@ -64,8 +67,20 @@ __ASM_STDCALL_FUNC( wrap_fastcall_func1, 8,
 
 DECLARE_CRITICAL_SECTION(wineusb_cs);
 
+static unixlib_handle_t unix_handle;
+
+static pthread_mutex_t unix_device_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static struct list unix_device_list = LIST_INIT(unix_device_list);
 static struct list device_list = LIST_INIT(device_list);
 
+struct unix_device
+{
+    struct list entry;
+
+    libusb_device_handle *handle;
+};
+
 struct usb_device
 {
     struct list entry;
@@ -73,8 +88,16 @@ struct usb_device
 
     DEVICE_OBJECT *device_obj;
 
-    libusb_device *libusb_device;
-    libusb_device_handle *handle;
+    /* Points to the parent USB device if this is a USB interface; otherwise
+     * NULL. */
+    struct usb_device *parent;
+    uint8_t interface_index;
+
+    uint8_t class, subclass, protocol;
+
+    uint16_t vendor, product, revision;
+
+    struct unix_device *unix_device;
 
     LIST_ENTRY irp_list;
 };
@@ -84,45 +107,77 @@ static DEVICE_OBJECT *bus_fdo, *bus_pdo;
 
 static libusb_hotplug_callback_handle hotplug_cb_handle;
 
-static void add_usb_device(libusb_device *libusb_device)
+static void destroy_unix_device(struct unix_device *unix_device)
+{
+    pthread_mutex_lock(&unix_device_mutex);
+    libusb_close(unix_device->handle);
+    list_remove(&unix_device->entry);
+    pthread_mutex_unlock(&unix_device_mutex);
+    free(unix_device);
+}
+
+static void add_usb_interface(struct usb_device *parent, const struct libusb_interface_descriptor *desc)
+{
+    struct usb_device *device;
+    DEVICE_OBJECT *device_obj;
+    NTSTATUS status;
+
+    if ((status = IoCreateDevice(driver_obj, sizeof(*device), NULL,
+            FILE_DEVICE_USB, FILE_AUTOGENERATED_DEVICE_NAME, FALSE, &device_obj)))
+    {
+        ERR("Failed to create device, status %#x.\n", status);
+        return;
+    }
+
+    device = device_obj->DeviceExtension;
+    device->device_obj = device_obj;
+    device->parent = parent;
+    device->unix_device = parent->unix_device;
+    device->interface_index = desc->bInterfaceNumber;
+    device->class = desc->bInterfaceClass;
+    device->subclass = desc->bInterfaceSubClass;
+    device->protocol = desc->bInterfaceProtocol;
+    device->vendor = parent->vendor;
+    device->product = parent->product;
+    device->revision = parent->revision;
+    InitializeListHead(&device->irp_list);
+
+    EnterCriticalSection(&wineusb_cs);
+    list_add_tail(&device_list, &device->entry);
+    LeaveCriticalSection(&wineusb_cs);
+}
+
+static void add_unix_device(struct unix_device *unix_device)
 {
     static const WCHAR formatW[] = {'\\','D','e','v','i','c','e','\\','U','S','B','P','D','O','-','%','u',0};
+    libusb_device *libusb_device = libusb_get_device(unix_device->handle);
+    struct libusb_config_descriptor *config_desc;
     struct libusb_device_descriptor device_desc;
     static unsigned int name_index;
-    libusb_device_handle *handle;
     struct usb_device *device;
     DEVICE_OBJECT *device_obj;
     UNICODE_STRING string;
     NTSTATUS status;
-    WCHAR name[20];
+    WCHAR name[26];
     int ret;
 
     libusb_get_device_descriptor(libusb_device, &device_desc);
 
-    TRACE("Adding new device %p, vendor %04x, product %04x.\n", libusb_device,
+    TRACE("Adding new device %p, vendor %04x, product %04x.\n", unix_device,
             device_desc.idVendor, device_desc.idProduct);
 
-    if ((ret = libusb_open(libusb_device, &handle)))
-    {
-        WARN("Failed to open device: %s\n", libusb_strerror(ret));
-        return;
-    }
-
     sprintfW(name, formatW, name_index++);
     RtlInitUnicodeString(&string, name);
     if ((status = IoCreateDevice(driver_obj, sizeof(*device), &string,
             FILE_DEVICE_USB, 0, FALSE, &device_obj)))
     {
         ERR("Failed to create device, status %#x.\n", status);
-        LeaveCriticalSection(&wineusb_cs);
-        libusb_close(handle);
         return;
     }
 
     device = device_obj->DeviceExtension;
     device->device_obj = device_obj;
-    device->libusb_device = libusb_ref_device(libusb_device);
-    device->handle = handle;
+    device->unix_device = unix_device;
     InitializeListHead(&device->irp_list);
 
     EnterCriticalSection(&wineusb_cs);
@@ -130,22 +185,92 @@ static void add_usb_device(libusb_device *libusb_device)
     device->removed = FALSE;
     LeaveCriticalSection(&wineusb_cs);
 
+    device->class = device_desc.bDeviceClass;
+    device->subclass = device_desc.bDeviceSubClass;
+    device->protocol = device_desc.bDeviceProtocol;
+    device->vendor = device_desc.idVendor;
+    device->product = device_desc.idProduct;
+    device->revision = device_desc.bcdDevice;
+
+    if (!(ret = libusb_get_active_config_descriptor(libusb_device, &config_desc)))
+    {
+        /* Create new devices for interfaces of composite devices.
+         *
+         * On Windows this is the job of usbccgp.sys, a separate driver that
+         * layers on top of the base USB driver. While we could take this
+         * approach as well, implementing usbccgp is a lot more work, whereas
+         * interface support is effectively built into libusb.
+         *
+         * FIXME: usbccgp does not create separate interfaces in some cases,
+         * e.g. when there is an interface association descriptor available.
+         */
+        if (config_desc->bNumInterfaces > 1)
+        {
+            uint8_t i;
+
+            for (i = 0; i < config_desc->bNumInterfaces; ++i)
+            {
+                const struct libusb_interface *interface = &config_desc->interface[i];
+
+                if (interface->num_altsetting != 1)
+                    FIXME("Interface %u has %u alternate settings; using the first one.\n",
+                            i, interface->num_altsetting);
+                add_usb_interface(device, &interface->altsetting[0]);
+            }
+        }
+        libusb_free_config_descriptor(config_desc);
+    }
+    else
+    {
+        ERR("Failed to get configuration descriptor: %s\n", libusb_strerror(ret));
+    }
+
     IoInvalidateDeviceRelations(bus_pdo, BusRelations);
 }
 
-static void remove_usb_device(libusb_device *libusb_device)
+static void add_usb_device(libusb_device *libusb_device)
+{
+    struct libusb_device_descriptor device_desc;
+    struct unix_device *unix_device;
+    int ret;
+
+    libusb_get_device_descriptor(libusb_device, &device_desc);
+
+    TRACE("Adding new device %p, vendor %04x, product %04x.\n", libusb_device,
+            device_desc.idVendor, device_desc.idProduct);
+
+    if (!(unix_device = calloc(1, sizeof(*unix_device))))
+        return;
+
+    if ((ret = libusb_open(libusb_device, &unix_device->handle)))
+    {
+        WARN("Failed to open device: %s\n", libusb_strerror(ret));
+        free(unix_device);
+        return;
+    }
+    pthread_mutex_lock(&unix_device_mutex);
+    list_add_tail(&unix_device_list, &unix_device->entry);
+    pthread_mutex_unlock(&unix_device_mutex);
+
+    add_unix_device(unix_device);
+}
+
+static void remove_unix_device(struct unix_device *unix_device)
 {
     struct usb_device *device;
 
-    TRACE("Removing device %p.\n", libusb_device);
+    TRACE("Removing device %p.\n", unix_device);
 
     EnterCriticalSection(&wineusb_cs);
     LIST_FOR_EACH_ENTRY(device, &device_list, struct usb_device, entry)
     {
-        if (device->libusb_device == libusb_device)
+        if (device->unix_device == unix_device)
         {
-            list_remove(&device->entry);
-            device->removed = TRUE;
+            if (!device->removed)
+            {
+                device->removed = TRUE;
+                list_remove(&device->entry);
+            }
             break;
         }
     }
@@ -154,6 +279,19 @@ static void remove_usb_device(libusb_device *libusb_device)
     IoInvalidateDeviceRelations(bus_pdo, BusRelations);
 }
 
+static void remove_usb_device(libusb_device *libusb_device)
+{
+    struct unix_device *unix_device;
+
+    TRACE("Removing device %p.\n", libusb_device);
+
+    LIST_FOR_EACH_ENTRY(unix_device, &unix_device_list, struct unix_device, entry)
+    {
+        if (libusb_get_device(unix_device->handle) == libusb_device)
+            remove_unix_device(unix_device);
+    }
+}
+
 static BOOL thread_shutdown;
 static HANDLE event_thread;
 
@@ -257,10 +395,26 @@ static NTSTATUS fdo_pnp(IRP *irp)
             CloseHandle(event_thread);
 
             EnterCriticalSection(&wineusb_cs);
+            /* Normally we unlink all devices either:
+             *
+             * - as a result of hot-unplug, which unlinks the device, and causes
+             *   a subsequent IRP_MN_REMOVE_DEVICE which will free it;
+             *
+             * - if the parent is deleted (at shutdown time), in which case
+             *   ntoskrnl will send us IRP_MN_SURPRISE_REMOVAL and
+             *   IRP_MN_REMOVE_DEVICE unprompted.
+             *
+             * But we can get devices hotplugged between when shutdown starts
+             * and now, in which case they'll be stuck in this list and never
+             * freed.
+             *
+             * FIXME: This is still broken, though. If a device is hotplugged
+             * and then removed, it'll be unlinked and never freed. */
             LIST_FOR_EACH_ENTRY_SAFE(device, cursor, &device_list, struct usb_device, entry)
             {
-                libusb_unref_device(device->libusb_device);
-                libusb_close(device->handle);
+                assert(!device->removed);
+                if (!device->parent)
+                    destroy_unix_device(device->unix_device);
                 list_remove(&device->entry);
                 IoDeleteDevice(device->device_obj);
             }
@@ -282,30 +436,71 @@ static NTSTATUS fdo_pnp(IRP *irp)
     return IoCallDriver(bus_pdo, irp);
 }
 
-static void get_device_id(const struct usb_device *device, WCHAR *buffer)
+struct string_buffer
 {
+    WCHAR *string;
+    size_t len;
+};
+
+static void WINAPIV append_id(struct string_buffer *buffer, const WCHAR *format, ...)
+{
+    __ms_va_list args;
+    WCHAR *string;
+    int len;
+
+    __ms_va_start(args, format);
+
+    len = _vsnwprintf(NULL, 0, format, args) + 1;
+    if (!(string = ExAllocatePool(PagedPool, (buffer->len + len) * sizeof(WCHAR))))
+    {
+        if (buffer->string)
+            ExFreePool(buffer->string);
+        buffer->string = NULL;
+        return;
+    }
+    if (buffer->string)
+    {
+        memcpy(string, buffer->string, buffer->len * sizeof(WCHAR));
+        ExFreePool(buffer->string);
+    }
+    _vsnwprintf(string + buffer->len, len, format, args);
+    buffer->string = string;
+    buffer->len += len;
+
+    __ms_va_end(args);
+}
+
+static const WCHAR emptyW[] = {0};
+
+static void get_device_id(const struct usb_device *device, struct string_buffer *buffer)
+{
+    static const WCHAR interface_formatW[] = {'U','S','B','\\','V','I','D','_','%','0','4','X',
+            '&','P','I','D','_','%','0','4','X','&','M','I','_','%','0','2','X',0};
     static const WCHAR formatW[] = {'U','S','B','\\','V','I','D','_','%','0','4','X',
             '&','P','I','D','_','%','0','4','X',0};
-    struct libusb_device_descriptor desc;
 
-    libusb_get_device_descriptor(device->libusb_device, &desc);
-    sprintfW(buffer, formatW, desc.idVendor, desc.idProduct);
+    if (device->parent)
+        append_id(buffer, interface_formatW, device->vendor, device->product, device->interface_index);
+    else
+        append_id(buffer, formatW, device->vendor, device->product);
 }
 
-static void get_hardware_ids(const struct usb_device *device, WCHAR *buffer)
+static void get_hardware_ids(const struct usb_device *device, struct string_buffer *buffer)
 {
+    static const WCHAR interface_formatW[] = {'U','S','B','\\','V','I','D','_','%','0','4','X',
+                '&','P','I','D','_','%','0','4','X','&','R','E','V','_','%','0','4','X','&','M','I','_','%','0','2','X',0};
     static const WCHAR formatW[] = {'U','S','B','\\','V','I','D','_','%','0','4','X',
                 '&','P','I','D','_','%','0','4','X','&','R','E','V','_','%','0','4','X',0};
-    struct libusb_device_descriptor desc;
 
-    libusb_get_device_descriptor(device->libusb_device, &desc);
-    buffer += sprintfW(buffer, formatW, desc.idVendor, desc.idProduct, desc.bcdDevice) + 1;
+    if (device->parent)
+        append_id(buffer, interface_formatW, device->vendor, device->product, device->revision, device->interface_index);
+    else
+        append_id(buffer, formatW, device->vendor, device->product, device->revision);
     get_device_id(device, buffer);
-    buffer += strlenW(buffer) + 1;
-    *buffer = 0;
+    append_id(buffer, emptyW);
 }
 
-static void get_compatible_ids(const struct usb_device *device, WCHAR *buffer)
+static void get_compatible_ids(const struct usb_device *device, struct string_buffer *buffer)
 {
     static const WCHAR prot_format[] = {'U','S','B','\\','C','l','a','s','s','_','%','0','2','x',
             '&','S','u','b','C','l','a','s','s','_','%','0','2','x',
@@ -314,44 +509,35 @@ static void get_compatible_ids(const struct usb_device *device, WCHAR *buffer)
             '&','S','u','b','C','l','a','s','s','_','%','0','2','x',0};
     static const WCHAR class_format[] = {'U','S','B','\\','C','l','a','s','s','_','%','0','2','x',0};
 
-    struct libusb_device_descriptor device_desc;
-
-    libusb_get_device_descriptor(device->libusb_device, &device_desc);
-
-    buffer += sprintfW(buffer, prot_format, device_desc.bDeviceClass, device_desc.bDeviceSubClass,
-            device_desc.bDeviceProtocol) + 1;
-    buffer += sprintfW(buffer, subclass_format, device_desc.bDeviceClass, device_desc.bDeviceSubClass) + 1;
-    buffer += sprintfW(buffer, class_format, device_desc.bDeviceClass) + 1;
-    *buffer = 0;
+    append_id(buffer, prot_format, device->class, device->subclass, device->protocol);
+    append_id(buffer, subclass_format, device->class, device->subclass);
+    append_id(buffer, class_format, device->class);
+    append_id(buffer, emptyW);
 }
 
-static NTSTATUS query_id(const struct usb_device *device, IRP *irp, BUS_QUERY_ID_TYPE type)
+static NTSTATUS query_id(struct usb_device *device, IRP *irp, BUS_QUERY_ID_TYPE type)
 {
-    WCHAR *id = NULL;
+    static const WCHAR instance_idW[] = {'0',0};
+    struct string_buffer buffer = {0};
+
+    TRACE("type %#x.\n", type);
 
     switch (type)
     {
         case BusQueryDeviceID:
-            if ((id = ExAllocatePool(PagedPool, 28 * sizeof(WCHAR))))
-                get_device_id(device, id);
+            get_device_id(device, &buffer);
             break;
 
         case BusQueryInstanceID:
-            if ((id = ExAllocatePool(PagedPool, 2 * sizeof(WCHAR))))
-            {
-                id[0] = '0';
-                id[1] = 0;
-            }
+            append_id(&buffer, instance_idW);
             break;
 
         case BusQueryHardwareIDs:
-            if ((id = ExAllocatePool(PagedPool, (28 + 37 + 1) * sizeof(WCHAR))))
-                get_hardware_ids(device, id);
+            get_hardware_ids(device, &buffer);
             break;
 
         case BusQueryCompatibleIDs:
-            if ((id = ExAllocatePool(PagedPool, (33 + 25 + 13 + 1) * sizeof(WCHAR))))
-                get_compatible_ids(device, id);
+            get_compatible_ids(device, &buffer);
             break;
 
         default:
@@ -359,10 +545,27 @@ static NTSTATUS query_id(const struct usb_device *device, IRP *irp, BUS_QUERY_ID
             return irp->IoStatus.Status;
     }
 
-    irp->IoStatus.Information = (ULONG_PTR)id;
+    if (!buffer.string)
+        return STATUS_NO_MEMORY;
+
+    irp->IoStatus.Information = (ULONG_PTR)buffer.string;
     return STATUS_SUCCESS;
 }
 
+static void remove_pending_irps(struct usb_device *device)
+{
+    LIST_ENTRY *entry;
+    IRP *irp;
+
+    while ((entry = RemoveHeadList(&device->irp_list)) != &device->irp_list)
+    {
+        irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.ListEntry);
+        irp->IoStatus.Status = STATUS_DELETE_PENDING;
+        irp->IoStatus.Information = 0;
+        IoCompleteRequest(irp, IO_NO_INCREMENT);
+    }
+}
+
 static NTSTATUS pdo_pnp(DEVICE_OBJECT *device_obj, IRP *irp)
 {
     IO_STACK_LOCATION *stack = IoGetCurrentIrpStackLocation(irp);
@@ -388,38 +591,31 @@ static NTSTATUS pdo_pnp(DEVICE_OBJECT *device_obj, IRP *irp)
         }
 
         case IRP_MN_START_DEVICE:
+            ret = STATUS_SUCCESS;
+            break;
+
         case IRP_MN_SURPRISE_REMOVAL:
+            EnterCriticalSection(&wineusb_cs);
+            remove_pending_irps(device);
+            if (!device->removed)
+            {
+                device->removed = TRUE;
+                list_remove(&device->entry);
+            }
+            LeaveCriticalSection(&wineusb_cs);
             ret = STATUS_SUCCESS;
             break;
 
         case IRP_MN_REMOVE_DEVICE:
-        {
-            LIST_ENTRY *entry;
+            assert(device->removed);
+            remove_pending_irps(device);
 
-            EnterCriticalSection(&wineusb_cs);
-            while ((entry = RemoveHeadList(&device->irp_list)) != &device->irp_list)
-            {
-                irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.ListEntry);
-                irp->IoStatus.Status = STATUS_DELETE_PENDING;
-                irp->IoStatus.Information = 0;
-                IoCompleteRequest(irp, IO_NO_INCREMENT);
-            }
-            LeaveCriticalSection(&wineusb_cs);
-
-            if (device->removed)
-            {
-                libusb_unref_device(device->libusb_device);
-                libusb_close(device->handle);
-
-                irp->IoStatus.Status = STATUS_SUCCESS;
-                IoCompleteRequest(irp, IO_NO_INCREMENT);
-                IoDeleteDevice(device->device_obj);
-                return STATUS_SUCCESS;
-            }
+            if (!device->parent)
+                destroy_unix_device(device->unix_device);
 
+            IoDeleteDevice(device->device_obj);
             ret = STATUS_SUCCESS;
             break;
-        }
 
         default:
             FIXME("Unhandled minor function %#x.\n", stack->MinorFunction);
@@ -458,7 +654,7 @@ static NTSTATUS usbd_status_from_libusb(enum libusb_transfer_status status)
     }
 }
 
-static void transfer_cb(struct libusb_transfer *transfer)
+static void LIBUSB_CALL transfer_cb(struct libusb_transfer *transfer)
 {
     IRP *irp = transfer->user_data;
     URB *urb = IoGetCurrentIrpStackLocation(irp)->Parameters.Others.Argument1;
@@ -548,6 +744,7 @@ static struct pipe get_pipe(HANDLE handle)
 static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
 {
     URB *urb = IoGetCurrentIrpStackLocation(irp)->Parameters.Others.Argument1;
+    libusb_device_handle *handle = device->unix_device->handle;
     struct libusb_transfer *transfer;
     int ret;
 
@@ -567,9 +764,12 @@ static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
             for (entry = mark->Flink; entry != mark; entry = entry->Flink)
             {
                 IRP *queued_irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.ListEntry);
+                struct usb_cancel_transfer_params params =
+                {
+                    .transfer = queued_irp->Tail.Overlay.DriverContext[0],
+                };
 
-                if ((ret = libusb_cancel_transfer(queued_irp->Tail.Overlay.DriverContext[0])) < 0)
-                    ERR("Failed to cancel transfer: %s\n", libusb_strerror(ret));
+                __wine_unix_call(unix_handle, unix_usb_cancel_transfer, &params);
             }
             LeaveCriticalSection(&wineusb_cs);
 
@@ -581,7 +781,7 @@ static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
             struct _URB_PIPE_REQUEST *req = &urb->UrbPipeRequest;
             struct pipe pipe = get_pipe(req->PipeHandle);
 
-            if ((ret = libusb_clear_halt(device->handle, pipe.endpoint)) < 0)
+            if ((ret = libusb_clear_halt(handle, pipe.endpoint)) < 0)
                 ERR("Failed to clear halt: %s\n", libusb_strerror(ret));
 
             return STATUS_SUCCESS;
@@ -600,12 +800,12 @@ static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
 
             if (pipe.type == UsbdPipeTypeBulk)
             {
-                libusb_fill_bulk_transfer(transfer, device->handle, pipe.endpoint,
+                libusb_fill_bulk_transfer(transfer, handle, pipe.endpoint,
                         req->TransferBuffer, req->TransferBufferLength, transfer_cb, irp, 0);
             }
             else if (pipe.type == UsbdPipeTypeInterrupt)
             {
-                libusb_fill_interrupt_transfer(transfer, device->handle, pipe.endpoint,
+                libusb_fill_interrupt_transfer(transfer, handle, pipe.endpoint,
                         req->TransferBuffer, req->TransferBufferLength, transfer_cb, irp, 0);
             }
             else
@@ -646,7 +846,7 @@ static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
                     LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_STANDARD | LIBUSB_RECIPIENT_DEVICE,
                     LIBUSB_REQUEST_GET_DESCRIPTOR, (req->DescriptorType << 8) | req->Index,
                     req->LanguageId, req->TransferBufferLength);
-            libusb_fill_control_transfer(transfer, device->handle, buffer, transfer_cb, irp, 0);
+            libusb_fill_control_transfer(transfer, handle, buffer, transfer_cb, irp, 0);
             transfer->flags = LIBUSB_TRANSFER_FREE_BUFFER | LIBUSB_TRANSFER_FREE_TRANSFER;
             ret = libusb_submit_transfer(transfer);
             if (ret < 0)
@@ -703,7 +903,7 @@ static NTSTATUS usb_submit_urb(struct usb_device *device, IRP *irp)
                     req->Value, req->Index, req->TransferBufferLength);
             if (!(req->TransferFlags & USBD_TRANSFER_DIRECTION_IN))
                 memcpy(buffer + LIBUSB_CONTROL_SETUP_SIZE, req->TransferBuffer, req->TransferBufferLength);
-            libusb_fill_control_transfer(transfer, device->handle, buffer, transfer_cb, irp, 0);
+            libusb_fill_control_transfer(transfer, handle, buffer, transfer_cb, irp, 0);
             transfer->flags = LIBUSB_TRANSFER_FREE_BUFFER | LIBUSB_TRANSFER_FREE_TRANSFER;
             ret = libusb_submit_transfer(transfer);
             if (ret < 0)
@@ -725,9 +925,21 @@ static NTSTATUS WINAPI driver_internal_ioctl(DEVICE_OBJECT *device_obj, IRP *irp
     ULONG code = stack->Parameters.DeviceIoControl.IoControlCode;
     struct usb_device *device = device_obj->DeviceExtension;
     NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    BOOL removed;
 
     TRACE("device_obj %p, irp %p, code %#x.\n", device_obj, irp, code);
 
+    EnterCriticalSection(&wineusb_cs);
+    removed = device->removed;
+    LeaveCriticalSection(&wineusb_cs);
+
+    if (removed)
+    {
+        irp->IoStatus.Status = STATUS_DELETE_PENDING;
+        IoCompleteRequest(irp, IO_NO_INCREMENT);
+        return STATUS_DELETE_PENDING;
+    }
+
     switch (code)
     {
         case IOCTL_INTERNAL_USB_SUBMIT_URB:
@@ -773,10 +985,20 @@ static void WINAPI driver_unload(DRIVER_OBJECT *driver)
 
 NTSTATUS WINAPI DriverEntry(DRIVER_OBJECT *driver, UNICODE_STRING *path)
 {
+    NTSTATUS status;
+    void *instance;
     int err;
 
     TRACE("driver %p, path %s.\n", driver, debugstr_w(path->Buffer));
 
+    RtlPcToFileHeader(DriverEntry, &instance);
+    if ((status = NtQueryVirtualMemory(GetCurrentProcess(), instance,
+            MemoryWineUnixFuncs, &unix_handle, sizeof(unix_handle), NULL)))
+    {
+        ERR("Failed to initialize Unix library, status %#x.\n", status);
+        return status;
+    }
+
     driver_obj = driver;
 
     if ((err = libusb_init(NULL)))
