@@ -208,6 +208,24 @@ NTSTATUS WINAPI wow64_NtGetWriteWatch( UINT *args )
 }
 
 
+/**********************************************************************
+ *           wow64_NtInitializeNlsFiles
+ */
+NTSTATUS WINAPI wow64_NtInitializeNlsFiles( UINT *args )
+{
+    ULONG *addr32 = get_ptr( &args );
+    LCID *lcid = get_ptr( &args );
+    LARGE_INTEGER *size = get_ptr( &args );
+
+    void *addr;
+    NTSTATUS status;
+
+    status = NtInitializeNlsFiles( addr_32to64( &addr, addr32 ), lcid, size );
+    if (!status) put_addr( addr32, addr );
+    return status;
+}
+
+
 /**********************************************************************
  *           wow64_NtLockVirtualMemory
  */
@@ -269,6 +287,40 @@ NTSTATUS WINAPI wow64_NtMapViewOfSection( UINT *args )
     return status;
 }
 
+/**********************************************************************
+ *           wow64_NtMapViewOfSectionEx
+ */
+NTSTATUS WINAPI wow64_NtMapViewOfSectionEx( UINT *args )
+{
+    HANDLE handle = get_handle( &args );
+    HANDLE process = get_handle( &args );
+    ULONG *addr32 = get_ptr( &args );
+    const LARGE_INTEGER *offset = get_ptr( &args );
+    ULONG *size32 = get_ptr( &args );
+    ULONG alloc = get_ulong( &args );
+    ULONG protect = get_ulong( &args );
+    MEM_EXTENDED_PARAMETER *params = get_ptr( &args );
+    ULONG params_count = get_ulong( &args );
+
+    void *addr;
+    SIZE_T size;
+    NTSTATUS status;
+
+    status = NtMapViewOfSectionEx( handle, process, addr_32to64( &addr, addr32 ), offset, size_32to64( &size, size32 ), alloc,
+            protect, params, params_count );
+    if (NT_SUCCESS(status))
+    {
+        SECTION_IMAGE_INFORMATION info;
+
+        if (!NtQuerySection( handle, SectionImageInformation, &info, sizeof(info), NULL ))
+        {
+            if (info.Machine == current_machine) init_image_mapping( addr );
+        }
+        put_addr( addr32, addr );
+        put_size( size32, size );
+    }
+    return status;
+}
 
 /**********************************************************************
  *           wow64_NtProtectVirtualMemory
@@ -473,6 +525,19 @@ NTSTATUS WINAPI wow64_NtUnmapViewOfSection( UINT *args )
 }
 
 
+/**********************************************************************
+ *           wow64_NtUnmapViewOfSectionEx
+ */
+NTSTATUS WINAPI wow64_NtUnmapViewOfSectionEx( UINT *args )
+{
+    HANDLE process = get_handle( &args );
+    void *addr = get_ptr( &args );
+    ULONG flags = get_ulong( &args );
+
+    return NtUnmapViewOfSectionEx( process, addr, flags );
+}
+
+
 /**********************************************************************
  *           wow64_NtWow64AllocateVirtualMemory64
  */
