@@ -50,6 +50,15 @@ NTSTATUS WINAPI wow64_NtGdiExtGetObjectW( UINT *args )
     return NtGdiExtGetObjectW( handle, count, buffer );
 }
 
+NTSTATUS WINAPI wow64_NtGdiGetDCDword( UINT *args )
+{
+    HDC hdc = get_handle( &args );
+    UINT method = get_ulong( &args );
+    DWORD *result = get_ptr( &args );
+
+    return NtGdiGetDCDword( hdc, method, result );
+}
+
 NTSTATUS WINAPI wow64_NtGdiGetDCObject( UINT *args )
 {
     HDC hdc = get_handle( &args );
@@ -58,6 +67,15 @@ NTSTATUS WINAPI wow64_NtGdiGetDCObject( UINT *args )
     return HandleToUlong( NtGdiGetDCObject( hdc, type ));
 }
 
+NTSTATUS WINAPI wow64_NtGdiGetDCPoint( UINT *args )
+{
+    HDC hdc = get_handle( &args );
+    UINT method = get_ulong( &args );
+    POINT *result = get_ptr( &args );
+
+    return NtGdiGetDCPoint( hdc, method, result );
+}
+
 NTSTATUS WINAPI wow64_NtGdiCreateBitmap( UINT *args )
 {
     INT width = get_ulong( &args );
@@ -385,13 +403,13 @@ NTSTATUS WINAPI wow64_NtGdiSetTextJustification( UINT *args )
 
 NTSTATUS WINAPI wow64_NtGdiHfontCreate( UINT *args )
 {
-    const ENUMLOGFONTEXDVW *enumex = get_ptr( &args );
+    const void *logfont = get_ptr( &args );
     ULONG unk2 = get_ulong( &args );
     ULONG unk3 = get_ulong( &args );
     ULONG unk4 = get_ulong( &args );
     void *data = get_ptr( &args );
 
-    return HandleToUlong( NtGdiHfontCreate( enumex, unk2, unk3, unk4, data ));
+    return HandleToUlong( NtGdiHfontCreate( logfont, unk2, unk3, unk4, data ));
 }
 
 NTSTATUS WINAPI wow64_NtGdiGetFontFileData( UINT *args )
@@ -584,13 +602,6 @@ NTSTATUS WINAPI wow64_NtGdiFlush( UINT *args )
     return NtGdiFlush();
 }
 
-NTSTATUS WINAPI wow64_NtGdiDdDDICloseAdapter( UINT *args )
-{
-    const D3DKMT_CLOSEADAPTER *desc = get_ptr( &args );
-
-    return NtGdiDdDDICloseAdapter( desc );
-}
-
 NTSTATUS WINAPI wow64_NtGdiDdDDICreateDevice( UINT *args )
 {
     struct
@@ -626,26 +637,6 @@ NTSTATUS WINAPI wow64_NtGdiDdDDICreateDevice( UINT *args )
     return status;
 }
 
-NTSTATUS WINAPI wow64_NtGdiDdDDIOpenAdapterFromDeviceName( UINT *args )
-{
-    struct
-    {
-        ULONG pDeviceName;
-        D3DKMT_HANDLE hAdapter;
-        LUID AdapterLuid;
-    } *desc32 = get_ptr( &args );
-
-    D3DKMT_OPENADAPTERFROMDEVICENAME desc = { UlongToPtr( desc32->pDeviceName ) };
-    NTSTATUS status;
-
-    if (!(status = NtGdiDdDDIOpenAdapterFromDeviceName( &desc )))
-    {
-        desc32->hAdapter = desc.hAdapter;
-        desc32->AdapterLuid = desc.AdapterLuid;
-    }
-    return status;
-}
-
 NTSTATUS WINAPI wow64_NtGdiDdDDIOpenAdapterFromHdc( UINT *args )
 {
     struct
@@ -668,13 +659,6 @@ NTSTATUS WINAPI wow64_NtGdiDdDDIOpenAdapterFromHdc( UINT *args )
     return status;
 }
 
-NTSTATUS WINAPI wow64_NtGdiDdDDIOpenAdapterFromLuid( UINT *args )
-{
-    D3DKMT_OPENADAPTERFROMLUID *desc = get_ptr( &args );
-
-    return NtGdiDdDDIOpenAdapterFromLuid( desc );
-}
-
 NTSTATUS WINAPI wow64_NtGdiDdDDIQueryStatistics( UINT *args )
 {
     D3DKMT_QUERYSTATISTICS *stats = get_ptr( &args );
