@@ -35,12 +35,12 @@ struct expr
     {
         struct
         {
-            INT_PTR             value;
+            dbg_lgint_t         value;
         } s_const;
 
         struct
         {
-            UINT_PTR            value;
+            dbg_lguint_t        value;
         } u_const;
 
         struct
@@ -62,7 +62,7 @@ struct expr
         {
             int                 unop_type;
             struct expr*        exp1;
-            INT_PTR             result;
+            dbg_lgint_t         result;
         } unop;
 
         struct
@@ -70,20 +70,20 @@ struct expr
             int                 binop_type;
             struct expr*        exp1;
             struct expr*        exp2;
-            INT_PTR             result;
+            dbg_lgint_t         result;
         } binop;
 
         struct
         {
-            struct type_expr_t  cast_to;
+            struct dbg_type     cast_to;
             struct expr*        expr;
+            dbg_lgint_t         result;
         } cast;
 
         struct
         {
             struct expr*        exp1;
             const char*         element_name;
-            ULONG               result;
         } structure;
 
         struct
@@ -91,7 +91,7 @@ struct expr
             const char*         funcname;
             int	                nargs;
             struct expr*        arg[5];
-            ULONG               result;
+            dbg_lguint_t        result;
         } call;
 
     } un;
@@ -129,14 +129,14 @@ void expr_free_all(void)
     next_expr_free = 0;
 }
 
-struct expr* expr_alloc_typecast(struct type_expr_t* tet, struct expr* exp)
+struct expr* expr_alloc_typecast(struct dbg_type* type, struct expr* exp)
 {
     struct expr*        ex;
 
     ex = expr_alloc();
 
     ex->type            = EXPR_TYPE_CAST;
-    ex->un.cast.cast_to = *tet;
+    ex->un.cast.cast_to = *type;
     ex->un.cast.expr    = exp;
     return ex;
 }
@@ -163,7 +163,7 @@ struct expr* expr_alloc_symbol(const char* name)
     return ex;
 }
 
-struct expr* expr_alloc_sconstant(INT_PTR value)
+struct expr* expr_alloc_sconstant(dbg_lgint_t value)
 {
     struct expr*        ex;
 
@@ -174,7 +174,7 @@ struct expr* expr_alloc_sconstant(INT_PTR value)
     return ex;
 }
 
-struct expr* expr_alloc_uconstant(UINT_PTR value)
+struct expr* expr_alloc_uconstant(dbg_lguint_t value)
 {
     struct expr*        ex;
 
@@ -273,89 +273,74 @@ struct expr* WINAPIV expr_alloc_func_call(const char* funcname, int nargs, ...)
 struct dbg_lvalue expr_eval(struct expr* exp)
 {
     struct dbg_lvalue                   rtn;
-    int		                        i;
     struct dbg_lvalue                   exp1;
     struct dbg_lvalue                   exp2;
     DWORD64	                        scale1, scale2, scale3;
     struct dbg_type                     type1, type2;
     DWORD                               tag;
     const struct dbg_internal_var*      div;
+    BOOL                                ret;
 
-    rtn.cookie       = 0;
-    rtn.type.id      = dbg_itype_none;
-    rtn.type.module  = 0;
-    rtn.addr.Mode    = AddrModeFlat;
-    rtn.addr.Offset  = 0;
-    rtn.addr.Segment = 0;
+    init_lvalue_in_debugger(&rtn, 0, dbg_itype_none, NULL);
 
     switch (exp->type)
     {
     case EXPR_TYPE_CAST:
-        /* this is really brute force, we simply change the type... without 
-         * checking if this is right or not
-         */
-        rtn = expr_eval(exp->un.cast.expr);
-        switch (exp->un.cast.cast_to.type)
-        {
-        case type_expr_type_id:
-            if (exp->un.cast.cast_to.u.type.id == dbg_itype_none)
-            {
-                dbg_printf("Can't cast to unknown type\n");
-                RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-            }
-            rtn.type = exp->un.cast.cast_to.u.type;
-            break;
-        case type_expr_udt_class:
-        case type_expr_udt_struct:
-        case type_expr_udt_union:
-            rtn.type = types_find_type(rtn.type.module, exp->un.cast.cast_to.u.name,
-                                       SymTagUDT);
-            if (rtn.type.id == dbg_itype_none)
-            {
-                dbg_printf("Can't cast to UDT %s\n", exp->un.cast.cast_to.u.name);
-                RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-            }
-            break;
-        case type_expr_enumeration:
-            rtn.type = types_find_type(rtn.type.module, exp->un.cast.cast_to.u.name,
-                                       SymTagEnum);
-            if (rtn.type.id == dbg_itype_none)
-            {
-                dbg_printf("Can't cast to enumeration %s\n", exp->un.cast.cast_to.u.name);
-                RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-            }
-            break;
-        default:
-            dbg_printf("Unsupported cast type %u\n", exp->un.cast.cast_to.type);
+        exp1 = expr_eval(exp->un.cast.expr);
+        if (exp1.type.id == dbg_itype_none)
             RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-        }
-        for (i = 0; i < exp->un.cast.cast_to.deref_count; i++)
+        rtn = exp1;
+        rtn.type = exp->un.cast.cast_to;
+        init_lvalue_in_debugger(&rtn, exp->un.cast.cast_to.module, exp->un.cast.cast_to.id, &exp->un.cast.result);
+        if (types_is_float_type(&exp1))
         {
-            rtn.type = types_find_pointer(&rtn.type);
-            if (rtn.type.id == dbg_itype_none)
+            double dbl;
+            ret = memory_fetch_float(&exp1, &dbl);
+            if (ret)
             {
-                dbg_printf("Cannot find pointer type\n");
-                RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
+                if (types_is_float_type(&rtn))
+                    ret = memory_store_float(&rtn, &dbl);
+                else if (types_is_integral_type(&rtn))
+                    ret = memory_store_integer(&rtn, (dbg_lgint_t)dbl);
+                else
+                    ret = FALSE;
             }
         }
+        else if (types_is_integral_type(&exp1))
+        {
+            dbg_lgint_t val = types_extract_as_integer(&exp1);
+            if (types_is_float_type(&rtn))
+            {
+                double dbl = val;
+                ret = memory_store_float(&rtn, &dbl);
+            }
+            else if (types_is_integral_type(&rtn) || types_is_pointer_type(&rtn))
+                ret = memory_store_integer(&rtn, val);
+            else
+                ret = FALSE;
+        }
+        else if (types_is_pointer_type(&exp1))
+        {
+            dbg_lgint_t val = types_extract_as_integer(&exp1);
+
+            if (types_is_integral_type(&rtn) || types_is_pointer_type(&rtn))
+                ret = memory_store_integer(&rtn, val);
+            else
+                ret = FALSE;
+        }
+        else
+            ret = FALSE;
+        if (!ret)
+            RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
         break;
     case EXPR_TYPE_STRING:
-        rtn.cookie      = DLV_HOST;
-        rtn.type.id     = dbg_itype_astring;
-        rtn.type.module = 0;
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.string.str;
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_astring, &exp->un.string.str);
         break;
     case EXPR_TYPE_U_CONST:
-        rtn.cookie      = DLV_HOST;
-        rtn.type.id     = dbg_itype_unsigned_long_int;
-        rtn.type.module = 0;
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.u_const.value;
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_lguint, &exp->un.u_const.value);
         break;
     case EXPR_TYPE_S_CONST:
-        rtn.cookie      = DLV_HOST;
-        rtn.type.id     = dbg_itype_signed_long_int;
-        rtn.type.module = 0;
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.s_const.value;
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_lgint, &exp->un.s_const.value);
         break;
     case EXPR_TYPE_SYMBOL:
         switch (symbol_get_lvalue(exp->un.symbol.name, -1, &rtn, FALSE))
@@ -375,8 +360,7 @@ struct dbg_lvalue expr_eval(struct expr* exp)
         if (exp1.type.id == dbg_itype_none || !types_array_index(&exp1, 0, &rtn) ||
             rtn.type.id == dbg_itype_none)
             RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-        if (!types_udt_find_element(&rtn, exp->un.structure.element_name,
-                                    &exp->un.structure.result))
+        if (!types_udt_find_element(&rtn, exp->un.structure.element_name))
         {
             dbg_printf("%s\n", exp->un.structure.element_name);
             RaiseException(DEBUG_STATUS_NO_FIELD, 0, 0, NULL);
@@ -386,8 +370,7 @@ struct dbg_lvalue expr_eval(struct expr* exp)
         exp1 = expr_eval(exp->un.structure.exp1);
         if (exp1.type.id == dbg_itype_none) RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
         rtn = exp1;
-        if (!types_udt_find_element(&rtn, exp->un.structure.element_name,
-                                    &exp->un.structure.result))
+        if (!types_udt_find_element(&rtn, exp->un.structure.element_name))
         {
             dbg_printf("%s\n", exp->un.structure.element_name);
             RaiseException(DEBUG_STATUS_NO_FIELD, 0, 0, NULL);
@@ -457,28 +440,22 @@ struct dbg_lvalue expr_eval(struct expr* exp)
          */
         exp->un.call.result = 0;
 #endif
-        rtn.cookie = DLV_HOST;
-        /* get return type from function signature tupe */
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_none, &exp->un.call.result);
+        /* get return type from function signature type */
+        /* FIXME rtn.type.module should be set to function's module... */
         types_get_info(&rtn.type, TI_GET_TYPE, &rtn.type.id);
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.call.result;
         break;
     case EXPR_TYPE_INTVAR:
-        rtn.cookie = DLV_HOST;
         if (!(div = dbg_get_internal_var(exp->un.intvar.name)))
             RaiseException(DEBUG_STATUS_NO_SYMBOL, 0, 0, NULL);
-        rtn.type.id     = div->typeid;
-        rtn.type.module = 0;
-        rtn.addr.Offset = (ULONG_PTR)div->pval;
+        init_lvalue_in_debugger(&rtn, 0, div->typeid, div->pval);
         break;
     case EXPR_TYPE_BINOP:
-        rtn.cookie = DLV_HOST;
         exp1 = expr_eval(exp->un.binop.exp1);
         exp2 = expr_eval(exp->un.binop.exp2);
         if (exp1.type.id == dbg_itype_none || exp2.type.id == dbg_itype_none)
             RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-        rtn.type.id = dbg_itype_signed_long_int;
-        rtn.type.module = 0;
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.binop.result;
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_lgint, &exp->un.binop.result);
         type1 = exp1.type;
         type2 = exp2.type;
         switch (exp->un.binop.binop_type)
@@ -506,8 +483,8 @@ struct dbg_lvalue expr_eval(struct expr* exp)
                 types_get_info(&type2, TI_GET_LENGTH, &scale1);
                 rtn.type = exp2.type;
 	    }
-            exp->un.binop.result = types_extract_as_integer(&exp1) * (DWORD)scale1 +
-                (DWORD)scale2 * types_extract_as_integer(&exp2);
+            exp->un.binop.result = types_extract_as_integer(&exp1) * (dbg_lguint_t)scale1 +
+                (dbg_lguint_t)scale2 * types_extract_as_integer(&exp2);
             break;
 	case EXP_OP_SUB:
             if (!types_get_info(&exp1.type, TI_GET_SYMTAG, &tag) ||
@@ -538,8 +515,8 @@ struct dbg_lvalue expr_eval(struct expr* exp)
                 types_get_info(&type2, TI_GET_LENGTH, &scale1);
                 rtn.type = exp2.type;
 	    }
-            exp->un.binop.result = (types_extract_as_integer(&exp1) * (DWORD)scale1 - 
-                                    types_extract_as_integer(&exp2) * (DWORD)scale2) / (DWORD)scale3;
+            exp->un.binop.result = (types_extract_as_integer(&exp1) * (dbg_lguint_t)scale1 -
+                                    types_extract_as_integer(&exp2) * (dbg_lguint_t)scale2) / (dbg_lguint_t)scale3;
             break;
 	case EXP_OP_SEG:
             rtn.type.id = dbg_itype_segptr;
@@ -581,10 +558,10 @@ struct dbg_lvalue expr_eval(struct expr* exp)
             exp->un.binop.result = (types_extract_as_integer(&exp1) != types_extract_as_integer(&exp2));
             break;
 	case EXP_OP_SHL:
-            exp->un.binop.result = ((UINT_PTR)types_extract_as_integer(&exp1) << types_extract_as_integer(&exp2));
+            exp->un.binop.result = types_extract_as_integer(&exp1) << types_extract_as_integer(&exp2);
             break;
 	case EXP_OP_SHR:
-            exp->un.binop.result = ((UINT_PTR)types_extract_as_integer(&exp1) >> types_extract_as_integer(&exp2));
+            exp->un.binop.result = types_extract_as_integer(&exp1) >> types_extract_as_integer(&exp2);
             break;
 	case EXP_OP_MUL:
             exp->un.binop.result = (types_extract_as_integer(&exp1) * types_extract_as_integer(&exp2));
@@ -605,12 +582,9 @@ struct dbg_lvalue expr_eval(struct expr* exp)
 	}
         break;
     case EXPR_TYPE_UNOP:
-        rtn.cookie = DLV_HOST;
         exp1 = expr_eval(exp->un.unop.exp1);
         if (exp1.type.id == dbg_itype_none) RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
-        rtn.addr.Offset = (ULONG_PTR)&exp->un.unop.result;
-        rtn.type.id     = dbg_itype_signed_long_int;
-        rtn.type.module = 0;
+        init_lvalue_in_debugger(&rtn, 0, dbg_itype_lgint, &exp->un.unop.result);
         switch (exp->un.unop.unop_type)
 	{
 	case EXP_OP_NEG:
@@ -626,18 +600,12 @@ struct dbg_lvalue expr_eval(struct expr* exp)
             if (!types_array_index(&exp1, 0, &rtn))
                 RaiseException(DEBUG_STATUS_BAD_TYPE, 0, 0, NULL);
             break;
-	case EXP_OP_FORCE_DEREF:
-            rtn = exp1;
-            if (exp1.cookie == DLV_TARGET)
-                dbg_read_memory(memory_to_linear_addr(&exp1.addr), &rtn.addr.Offset, sizeof(rtn.addr.Offset));
-            break;
 	case EXP_OP_ADDR:
             /* only do it on linear addresses */
             if (exp1.addr.Mode != AddrModeFlat)
                 RaiseException(DEBUG_STATUS_CANT_DEREF, 0, 0, NULL);
             exp->un.unop.result = (ULONG_PTR)memory_to_linear_addr(&exp1.addr);
-            rtn.type = types_find_pointer(&exp1.type);
-            if (rtn.type.id == dbg_itype_none)
+            if (!types_find_pointer(&exp1.type, &rtn.type))
                 RaiseException(DEBUG_STATUS_CANT_DEREF, 0, 0, NULL);
             break;
 	default: RaiseException(DEBUG_STATUS_INTERNAL_ERROR, 0, 0, NULL);
@@ -655,30 +623,12 @@ struct dbg_lvalue expr_eval(struct expr* exp)
 BOOL expr_print(const struct expr* exp)
 {
     int		        i;
-    struct dbg_type     type;
 
     switch (exp->type)
     {
     case EXPR_TYPE_CAST:
-        WINE_FIXME("No longer supported (missing module base)\n");
         dbg_printf("((");
-        switch (exp->un.cast.cast_to.type)
-        {
-        case type_expr_type_id:
-            type.module = 0;
-            type.id = exp->un.cast.cast_to.type;
-            types_print_type(&type, FALSE); break;
-        case type_expr_udt_class:
-            dbg_printf("class %s", exp->un.cast.cast_to.u.name); break;
-        case type_expr_udt_struct:
-            dbg_printf("struct %s", exp->un.cast.cast_to.u.name); break;
-        case type_expr_udt_union:
-            dbg_printf("union %s", exp->un.cast.cast_to.u.name); break;
-        case type_expr_enumeration:
-            dbg_printf("enum %s", exp->un.cast.cast_to.u.name); break;
-        }
-        for (i = 0; i < exp->un.cast.cast_to.deref_count; i++)
-            dbg_printf("*");
+        types_print_type(&exp->un.cast.cast_to, FALSE);
         dbg_printf(")");
         expr_print(exp->un.cast.expr);
         dbg_printf(")");
@@ -687,10 +637,10 @@ BOOL expr_print(const struct expr* exp)
         dbg_printf("$%s", exp->un.intvar.name);
         break;
     case EXPR_TYPE_U_CONST:
-        dbg_printf("%Iu", exp->un.u_const.value);
+        dbg_printf("%I64u", exp->un.u_const.value);
         break;
     case EXPR_TYPE_S_CONST:
-        dbg_printf("%Id", exp->un.s_const.value);
+        dbg_printf("%I64d", exp->un.s_const.value);
         break;
     case EXPR_TYPE_STRING:
         dbg_printf("\"%s\"", exp->un.string.str);
