@@ -114,8 +114,8 @@ BOOL memory_write_value(const struct dbg_lvalue* lvalue, DWORD size, void* value
     if (!types_get_info(&lvalue->type, TI_GET_LENGTH, &os)) return FALSE;
     if (size != os)
     {
-        dbg_printf("Size mismatch in memory_write_value, got %lu from type while expecting %lu\n",
-                   (DWORD)os, size);
+        dbg_printf("Size mismatch in memory_write_value, got %I64u from type while expecting %lu\n",
+                   os, size);
         return FALSE;
     }
 
@@ -452,8 +452,7 @@ char* memory_offset_to_string(char *str, DWORD64 offset, unsigned mode)
     if (mode == 32)
         sprintf(str, "0x%08x", (unsigned int) offset);
     else
-        sprintf(str, "0x%08x%08x", (unsigned int)(offset >> 32),
-                (unsigned int)offset);
+        sprintf(str, "%#016I64x", offset);
 
     return str;
 }
@@ -501,7 +500,6 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
         case btInt:
         case btLong:
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
-            if (size == 1) goto print_char;
             dbg_print_hex(size, val_int);
             break;
         case btUInt:
@@ -514,17 +512,18 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
             dbg_printf("%f", val_real);
             break;
         case btChar:
-        case btWChar:
-            /* sometimes WCHAR is defined as btChar with size = 2, so discrimate
-             * Ansi/Unicode based on size, not on basetype
-             */
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
-        print_char:
-            if ((size == 1 && isprint((char)val_int)) ||
-                 (size == 2 && val_int < 127 && isprint((char)val_int)))
+            if (size == 1 && isprint((char)val_int))
                 dbg_printf("'%c'", (char)val_int);
             else
-                dbg_printf("%d", (int)val_int);
+                dbg_print_hex(size, val_int);
+            break;
+        case btWChar:
+            if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
+            if (size == 2 && iswprint((WCHAR)val_int))
+                dbg_printf("L'%lc'", (WCHAR)val_int);
+            else
+                dbg_print_hex(size, val_int);
             break;
         case btBool:
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
@@ -550,11 +549,11 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
             char    buffer[1024];
 
             if (!val_ptr) dbg_printf("0x0");
-            else if (((bt == btChar || bt == btInt) && size64 == 1) || (bt == btUInt && size64 == 2))
+            else if ((bt == btChar && size64 == 1) || (bt == btWChar && size64 == 2))
             {
                 if (memory_get_string(dbg_curr_process, val_ptr, sub_lvalue.in_debuggee,
-                                      size64 == 2, buffer, sizeof(buffer)))
-                    dbg_printf("\"%s\"", buffer);
+                                      bt == btWChar, buffer, sizeof(buffer)))
+                    dbg_printf("%s\"%s\"", bt == btWChar ? "L" : "", buffer);
                 else
                     dbg_printf("*** invalid address %p ***", val_ptr);
                 break;
@@ -641,7 +640,6 @@ void print_basic(const struct dbg_lvalue* lvalue, char format)
     {
         unsigned size;
         dbg_lgint_t res = types_extract_as_lgint(lvalue, &size, NULL);
-        WCHAR wch;
 
         switch (format)
         {
@@ -658,10 +656,7 @@ void print_basic(const struct dbg_lvalue* lvalue, char format)
             return;
 
         case 'u':
-            wch = (WCHAR)(res & 0xFFFF);
-            dbg_printf("%d = '", wch);
-            dbg_outputW(&wch, 1);
-            dbg_printf("'");
+            dbg_printf("%d = '%lc'", (WCHAR)(res & 0xFFFF), (WCHAR)(res & 0xFFFF));
             return;
 
         case 'i':
@@ -788,7 +783,7 @@ void memory_disassemble(const struct dbg_lvalue* xstart,
         memory_disasm_one_insn(&last);
 }
 
-BOOL memory_get_register(DWORD regno, DWORD_PTR** value, char* buffer, int len)
+BOOL memory_get_register(DWORD regno, struct dbg_lvalue* lvalue, char* buffer, int len)
 {
     const struct dbg_internal_var*  div;
 
@@ -818,7 +813,7 @@ BOOL memory_get_register(DWORD regno, DWORD_PTR** value, char* buffer, int len)
     {
         if (div->val == regno)
         {
-            if (!stack_get_register_frame(div, value))
+            if (!stack_get_register_frame(div, lvalue))
             {
                 if (buffer) snprintf(buffer, len, "<register %s not accessible in this frame>", div->name);
                 return FALSE;
